import luigi
import luigi.contrib.gcs
import luigi.format
import luigi.contrib.opener

class GCSOpener(luigi.contrib.opener.Opener):
    """Opens a target stored on Google GCS

    examples:
    * gs://bucket/foo/bar.txt

    """
    names = ['gs']
    allowed_kwargs = {
        'format': True,
        'client': True,
    }
    filter_kwargs = False

    @classmethod
    def get_target(cls, scheme, path, fragment, username,
                   password, hostname, port, query, **kwargs):
        import luigi.contrib.gcs
        query.update(kwargs)
        return luigi.contrib.gcs.GCSTarget('{scheme}://{hostname}{path}'.format(
            scheme=scheme, hostname=hostname, path=path), **query)
    
luigi.contrib.opener.opener.add(GCSOpener)



import requests
import urllib.parse
import json
import yaml
import os

class LuigiTaskClient(object):
    def __init__(self, task_name, scheduler_url=None, **task_params):
        self.task_name = task_name
        self.task_params = task_params
        self._scheduler_url = scheduler_url
        self.tasks = {}

    def _match_task_params(self, **params):
        for key, value in self.task_params.items():
            if params.get(key, None) != value:
                return False
        return True

    @property
    def scheduler_url(self):
        if self._scheduler_url is not None:
            return self._scheduler_url
        return os.environ.get("SCHEDULER_URL", "http://127.0.0.1:8082")

    def get_tasks(self, all=False):
        if not self.tasks or all:
            tasks = requests.get(
                self.scheduler_url
                + "/api/task_list?data=" +
                urllib.parse.quote(json.dumps({
                    "upstream_status":"","search": self.task_name}))
            ).json()

            for taskid, task in tasks["response"].items():
                if task['name'] != self.task_name: continue
                if not self._match_task_params(**task['params']):
                    continue
                task["id"] = taskid
                self.tasks[taskid] = task
        return self.tasks
    
    def _get_logs(self, all=False):
        tasks = self.get_tasks(all)
        logs = []
        for task_id in tasks.keys():
            status = requests.get(
                os.environ.get("SCHEDULER_URL", "http://127.0.0.1:8082")
                + "/api/get_task_status_message?data=" + urllib.parse.quote(json.dumps({"task_id": task_id}))
            ).json()
            logs.append(status["response"]["statusMessage"])
        return logs

    def get_logs(self, all=False):
        return [log.split("\n") if log else [] for log in self._get_logs()]
    
    def get_logs_yaml(self):
        return [yaml.load(log, Loader=yaml.SafeLoader) if log else [] for log in self._get_logs()]




#!/usr/bin/env python

import setuptools

setuptools.setup(
    name='poltergust-luigi-utils',
    version='0.0.3',
    description='Luigi utils',
    long_description='''Luigi utils for use together with Poltergust
''',
    long_description_content_type="text/markdown",
    author='Egil Moeller',
    author_email='em@emrld.no',
    url='https://github.com/emerald-geomodelling/poltergust-luigi-utils',
    packages=setuptools.find_packages(),
    install_requires=[
        "luigi",
        "requests",
        "google-auth",
        "google-api-python-client",
    ],
)



import yaml
import luigi.contrib.opener
from . import gcs_opener
from . import luigi_client

# Valid values for status is
# None: Not started yet
# "running": The pipeline is currently running
# "done": The pipeline succeeded
# "failed": The pipeline failed

class PoltergustPipelineClient(object):
    def __init__(self,
                 pipeline_url = None,
                 
                 task_name = None,
                 task_params = {},

                 name = None,
                 output_url = None,
                 log_url = None,
    
                 environment = None,
                 variables = {},

                 status = None,
                 set_status = lambda status: status):

        self.pipeline_url = pipeline_url or os.environ.get("PIPELINE_URL", "./pipelines")
        
        self.name = name
        self.output_url = output_url
        self.log_url = log_url

        self.task_name = task_name
        self.task_params = task_params

        self.environment = environment
        self.variables = variables
    
        self._status = status
        self.set_status = set_status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self.set_status(value)
        self._status = value
        
    @property
    def url(self):
        return "%s.config.yaml" % (self.base_url,)

    @property
    def base_url(self):
        return "%s/%s" % (self.pipeline_url, self.name)

    @property
    def _task_name(self):
        return self.task_name.rsplit(".", 1)[1]
    
    @property
    def _task_module(self):
        return self.task_name.rsplit(".", 1)[0]
    
    @property
    def config(self):
        task = {
            "name": self._task_name,
            "module": self._task_module,
        }
        task.update(self.task_params)
        return {
            "environment": self.environment,
            "task": task,
            "variables": self.variables
        }
    
    def launch(self):
        with luigi.contrib.opener.OpenerTarget(self.url).open("w") as f:
            yaml.dump(self.config, f)

        self.status = "running"

    def cancel(self):
        luigi.contrib.opener.OpenerTarget(self.url).remove()
        
        self.status = None

    def get_loggers(self):
        if not hasattr(self, "loggers"):
            self.loggers = {
                "MakeEnvironment": luigi_client.LuigiTaskClient("MakeEnvironment", path=self.environment),
                "RunTask": luigi_client.LuigiTaskClient("RunTask", path=self.base_url),
                "Task":luigi_client.LuigiTaskClient(
                    self._task_name,
                    **{key.replace("-", "_"): value for key, value in self.task_params.items()}),
            }
        return self.loggers

    def update_status(self):
        if luigi.contrib.opener.OpenerTarget(self.output_url).exists():
            self.status = "done"
        elif luigi.contrib.opener.OpenerTarget("%s.error.yaml" % (self.base_url,)).exists():
            self.status = "failed"
    
    def get_log(self):
        logs = self.get_logs()
        return [logitem for log in logs.values() for logitem in log]

    @property
    def log_target(self):
        return luigi.contrib.opener.OpenerTarget(self.log_url)
    
    def get_logs(self):
        def flatten_logs(logs):
            return [logitem for log in logs for logitem in log]
        res = {}
        if self.status == "running":
            loggers = self.get_loggers()
            res["MakeEnvironment"] = flatten_logs(self.get_loggers()["MakeEnvironment"].get_logs())
            res["RunTask"] = flatten_logs(self.get_loggers()["RunTask"].get_logs())
            res["Task"] = flatten_logs(self.get_loggers()["Task"].get_logs_yaml())
        elif self.status is not None:
            try:
                with self.log_target.open() as f:
                    res["Task"] = yaml.load(f, Loader=yaml.SafeLoader)
            except:
                pass
            res["RunTask"] = []
            for url in self.log_target.fs.list_wildcard("%s.*.log.txt" % (self.base_url,)):
                with luigi.contrib.opener.OpenerTarget(url).open() as f:
                    res["RunTask"].append(f.read().split("\n"))
            res["RunTask"] = flatten_logs(res["RunTask"])
        #elif self.status is None:
        #    pass
            
        return res
